//	ListFileView.cpp - FileView displaying files in Windows Explorer's list
//					   style.
//	--------------------------------------------------------------------------
//	This file is part of 'Phil', a very simple file manager/browser for the
//	JUCE library.
//	Copyright (C) 2005-2006  Niall Moody
//	
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//	
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//	
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//	
//	--------------------------------------------------------------------------

#include <juce.h>
#include "ListFileView.h"
#include "PhilComponent.h"

//----------------------------------------------------------------------------
ListFileView::ListFileView(Phil *parent):
FileView(parent),
DragAndDropContainer(),
numColumnItems(1),
columnWidth(194),
previousFile(0)
{
	setWantsKeyboardFocus(true);
}

//----------------------------------------------------------------------------
ListFileView::~ListFileView()
{
	
}

//----------------------------------------------------------------------------
void ListFileView::paint(Graphics& g)
{
	long i, j;
	long row = 0;
	long column = 0;
	OwnedArray<File> children;
	Phil *parent = getPhil();
	const StringArray& highlights = parent->getCurrentlyHighlightedFiles();
	const Image& dirImage = FileView::getDefaultDirImage();
	const Image& fileImage = FileView::getDefaultFileImage();

	//First draw all the directories.
	parent->findChildFiles(children, true);

	for(i=0;i<children.size();++i)
	{
		//Check if this directory should be highlighted.
		//Hmm...  Is there a faster way to do this?
		for(j=0;j<highlights.size();++j)
		{
			if(highlights[j] == children[i]->getFullPathName())
			{
				//Draw the highlight.
				g.setColour(parent->getHighlightColour());
				g.fillRoundedRectangle((column*columnWidth),
									   (row*22),
									   columnWidth,
									   22,
									   5);
				break;
			}
		}

		//Draw the icon for the directory.
		g.drawImageAt(&dirImage, (column*columnWidth)+4, (row*22)+4);
		//Draw the name of the directory.
		g.setColour(parent->getTextColour());
		g.drawText(children[i]->getFileName(),
				   (column*columnWidth)+(20+3),
				   (row*22)+3,
				   (columnWidth-(20+6)),
				   16,
				   Justification::centredLeft,
				   true);

		++row;
		if(row > numColumnItems)
		{
			row = 0;
			++column;
		}
	}

	//Now draw all the files.
	children.clear();
	parent->findChildFiles(children, false);

	for(i=0;i<children.size();++i)
	{
		//Check if this file should be highlighted.
		//Hmm...  Is there a faster way to do this?
		for(j=0;j<highlights.size();++j)
		{
			if(highlights[j] == children[i]->getFullPathName())
			{
				//Draw the highlight.
				g.setColour(parent->getHighlightColour());
				g.fillRoundedRectangle((column*columnWidth),
									   (row*22),
									   columnWidth,
									   22,
									   5);
				break;
			}
		}

		//Draw the icon for the file.
		g.drawImageAt(&fileImage, (column*columnWidth)+4, (row*22)+4);
		//Draw the name of the file.
		g.setColour(parent->getTextColour());
		g.drawText(children[i]->getFileName(),
				   (column*columnWidth)+(20+3),
				   (row*22)+3,
				   (columnWidth-(20+6)),
				   16,
				   Justification::centredLeft,
				   true);

		++row;
		if(row > numColumnItems)
		{
			row = 0;
			++column;
		}
	}
}

//----------------------------------------------------------------------------
void ListFileView::mouseDown(const MouseEvent &e)
{
	long selectedIndex;
	Phil *parent = getPhil();

	selectedIndex = getClickedOnIndex(e.x, e.y);

	if(e.mods.isRightButtonDown())
		parent->displayContextMenuNow(selectedIndex, e.x, e.y);
	else if(!e.mods.isCommandDown())
		parent->fileSelected(selectedIndex);
	else if(selectedIndex != previousFile)
		parent->anotherFileSelected(selectedIndex);
	previousFile = selectedIndex;
}

//----------------------------------------------------------------------------
void ListFileView::mouseDrag(const MouseEvent &e)
{
	long selectedIndex;
	OwnedArray<File> children;
	Phil *parent = getPhil();

	selectedIndex = getClickedOnIndex(e.x, e.y);

	//Selecting multiple files.
	if(e.mods.isCommandDown())
	{
		if(selectedIndex != previousFile)
		{
			parent->anotherFileSelected(selectedIndex);
			previousFile = selectedIndex;
		}
	}
	//Moving a file(s).
	else if(parent->getDragAndDropAllowed())
	{
		//Check directories
		parent->findChildFiles(children, true);
		if(selectedIndex >= children.size())
		{
			//Check files.
			selectedIndex -= children.size();
			children.clear();
			parent->findChildFiles(children, false);
		}
		if(selectedIndex < children.size())
		{
			//See paint() for this stuff.
			Image *tempIm = new Image(Image::RGB, columnWidth, 22, true);
			Graphics tempG(*tempIm);
			tempG.setColour(parent->getHighlightColour());
			tempG.fillRoundedRectangle(0,
									   0,
									   columnWidth,
									   22,
									   5);
			//Draw the icon for the file/directory.
			if(children[selectedIndex]->isDirectory())
				tempG.drawImageAt(&(FileView::getDefaultDirImage()), 4, 4);
			else
				tempG.drawImageAt(&(FileView::getDefaultFileImage()), 4, 4);
			//Draw the name of the file/directory.
			tempG.setColour(parent->getTextColour());
			tempG.drawText(children[selectedIndex]->getFileName(),
						  (20+3),
						  3,
						  (columnWidth-(20+6)),
						  16,
						  Justification::centredLeft,
						  true);
			tempIm->multiplyAllAlphas(0.65f);
			//We just drag the file name, and don't actually use the Component
			//variable.
			startDragging(children[selectedIndex]->getFullPathName(),
						  this,
						  tempIm,
						  true);
		}
	}
}

//----------------------------------------------------------------------------
void ListFileView::mouseDoubleClick(const MouseEvent &e)
{
	long numFiles;
	long selectedIndex;
	OwnedArray<File> children;
	Phil *parent = getPhil();

	selectedIndex = getClickedOnIndex(e.x, e.y);

	numFiles = parent->getNumberOfChildFiles(true);
	numFiles += parent->getNumberOfChildFiles(false);

	if(selectedIndex < numFiles)
		parent->enterObject();
}

//----------------------------------------------------------------------------
bool ListFileView::isInterestedInDragSource(const String &sourceDescription)
{
	Phil *parent = getPhil();
	File test(sourceDescription);

	if(parent->getDragAndDropAllowed())
		return test.exists();
	else
		return false;
}

//----------------------------------------------------------------------------
void ListFileView::itemDropped(const String &sourceDescription,
							   Component *sourceComponent,
							   int x,
							   int y)
{
	long tempint;
	long selectedIndex;
	OwnedArray<File> children;
	Phil *parent = getPhil();

	if(!parent->getDragAndDropAllowed())
		return;

	selectedIndex = getClickedOnIndex(x, y);

	//Move to selected directory.
	parent->findChildFiles(children, true);
	if(selectedIndex < children.size())
	{
		if(sourceDescription != children[selectedIndex]->getFullPathName())
		{
			//We know sourceDescription is valid thanks to
			//isInterestedInDragSource().
			File movingFile(sourceDescription);
			movingFile.moveFileTo(children[selectedIndex]->getChildFile(movingFile.getFileName()));
			parent->repaint();

			//Make sure everything's deselected after.
			tempint = children.size();
			tempint += parent->getNumberOfChildFiles(false);
			parent->fileSelected(tempint+1);
		}
	}
	//Move to current directory, from another Phil app.
	else if(sourceComponent != this)
	{
		File movingFile(sourceDescription);
		movingFile.moveFileTo(parent->getCurrentDirectory().getChildFile(movingFile.getFileName()));
		parent->repaint();

		//Make sure everything's deselected after.
		tempint = children.size();
		tempint += parent->getNumberOfChildFiles(false);
		parent->fileSelected(tempint+1);
	}
}

//----------------------------------------------------------------------------
void ListFileView::itemDragMove(const String &sourceDescription,
								Component *sourceComponent,
								int x,
								int y)
{
	long selectedIndex;
	OwnedArray<File> children;
	Phil *parent = getPhil();

	selectedIndex = getClickedOnIndex(x, y);

	//Highlight any directories we pass over.
	parent->findChildFiles(children, true);
	if(selectedIndex < children.size())
		parent->fileSelected(selectedIndex);
}

//----------------------------------------------------------------------------
void ListFileView::keyPressed(const KeyPress &key)
{
	Phil *parent = getPhil();
	//const StringArray& selectedFiles = parent->getCurrentlyHighlightedFiles();

	//Modifier keys == shortcuts, handled by Phil.
	if((key.getModifiers().getRawFlags() == 0) &&
	   (key.getKeyCode() != KeyPress::upKey) &&
	   (key.getKeyCode() != KeyPress::downKey) &&
	   (key.getKeyCode() != KeyPress::leftKey) &&
	   (key.getKeyCode() != KeyPress::rightKey) &&
	   (key.getKeyCode() != KeyPress::spaceKey) &&
	   (key.getKeyCode() != KeyPress::returnKey) &&
	   (key.getKeyCode() != KeyPress::deleteKey))
	{
		StringComparator<File *> comp;
		char tempch = static_cast<char>(key.getKeyCode());

		//Things are simpler if there isn't anything selected at the moment.
		//if(selectedFiles.size() < 1)
		{
			long index = 0;
			String tempstr;
			OwnedArray<File> children;

			tempstr << tempch << T("*");
			parent->getCurrentDirectory().findChildFiles(children,
													   true,
													   false,
													   tempstr);
			children.sort(comp);
			//i.e. there aren't any directories that match.
			if(children.size() < 1)
			{
				parent->getCurrentDirectory().findChildFiles(children,
														   false,
														   false,
														   tempstr);
				children.sort(comp);
				//Found a file/some files which match.
				if(children.size() > 0)
				{
					File tempFil(*(children[0]));
					children.clear();

					//Calculate the index of the file to select via
					//Phil->fileSelected().
					parent->findChildFiles(children, false);
					while(*(children[index]) != tempFil)
						++index;
					index += parent->getNumberOfChildFiles(true);

					parent->fileSelected(index);
				}
			}
			//Found a directory/some directories which match.
			else
			{
				File tempFil(*(children[0]));
				children.clear();

				//Calculate the index of the directory to select via
				//Phil->fileSelected().
				parent->findChildFiles(children, true);
				while(*(children[index]) != tempFil)
					++index;

				parent->fileSelected(index);
			}
		}
		/*
		//There's already a file/some files selected.
		//In this case, we need to work out if the currently-selected file
		//has the same starting character as the one we're looking for, and if
		//so, select the *next* file with that starting character.
		else
		{
			long oldIndex = 0;
			long index = 0;
			String tempstr;
			OwnedArray<File> children;
			File currentFile(selectedFiles[0]);

			//The currently-selected file starts with the same character we're
			//looking for.
			if(selectedFiles[0].startsWithIgnoreCase(tempch))
			{
				bool foundDirectory = false;
				parent->findChildFiles(children, true);

				for(oldIndex=0;oldIndex<children.size();++oldIndex)
				{
					if(children[oldIndex] == currentFile)
					{
						foundDirectory = true;
						break;
					}
				}

				//
				if(foundDirectory)
				{
					
				}
			}
		}*/
	}

	Component::keyPressed(key);
}

//----------------------------------------------------------------------------
void ListFileView::updateSize(int width, int height, long numChildren)
{
	int i;
	int row = 0;
	int column = 1;

	//Height of a single file/directory is always 22 pixels.
	//Probably want to change this in future to something that's not constant.
	numColumnItems = (height/22)-1;

	for(i=0;i<numChildren;++i)
	{
		++row;
		if(row > numColumnItems)
		{
			row = 0;
			++column;
		}
	}
	setSize((column*columnWidth),
			height);
}

//----------------------------------------------------------------------------
const Rectangle& ListFileView::getSelectedBounds() const
{
	static Rectangle retval;

	long i, j;
	long row = 0;
	long column = 0;
	OwnedArray<File> children;
	Phil *parent = getPhil();
	const StringArray& highlights = parent->getCurrentlyHighlightedFiles();

	if(highlights.size() < 1)
		retval.setBounds(0, 0, 0, 0);
	else
	{
		//First go through all the directories.
		parent->findChildFiles(children, true);

		for(i=0;i<children.size();++i)
		{
			//Check if this directory should be highlighted.
			for(j=0;j<highlights.size();++j)
			{
				if(highlights[j] == children[i]->getFullPathName())
				{
					retval.setBounds((column*columnWidth),
									 (row*22),
									 columnWidth,
									 22);
					return retval;
				}
			}
			++row;
			if(row > numColumnItems)
			{
				row = 0;
				++column;
			}
		}

		//Now go through all the files.
		children.clear();
		parent->findChildFiles(children, false);
	
		for(i=0;i<children.size();++i)
		{
			//Check if this file should be highlighted.
			for(j=0;j<highlights.size();++j)
			{
				if(highlights[j] == children[i]->getFullPathName())
				{
					retval.setBounds((column*columnWidth),
									 (row*22),
									 columnWidth,
									 22);
					return retval;
				}
			}
			++row;
			if(row > numColumnItems)
			{
				row = 0;
				++column;
			}
		}
	}

	return retval;
}

//----------------------------------------------------------------------------
long ListFileView::getClickedOnIndex(int x, int y)
{
	long retval;

	retval = (((x/columnWidth)*(numColumnItems+1)) + (y/22));

	return retval;
}
